package net.sf.twip.mockito;

import org.junit.runners.model.FrameworkMethod;
import org.junit.runners.model.Statement;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import net.sf.twip.TwipExtension;

/**
 * Allows you to use the {@link org.mockito.Mockito mockito} {@link Mock @Mock}
 * annotation.
 */
public class MockitoExtension implements TwipExtension {

	private static final class MockitoStatement extends Statement {
		private final Statement statement;
		private final Object testInstance;

		public MockitoStatement(Statement statement, Object testInstance) {
			this.statement = statement;
			this.testInstance = testInstance;
		}

		@Override
		public void evaluate() throws Throwable {
			MockitoAnnotations.initMocks(testInstance);
			statement.evaluate();
			Mockito.validateMockitoUsage();
		}
	}

	public Statement wrap(FrameworkMethod method, Object testInstance, Statement statement) {
		return new MockitoStatement(statement, testInstance);
	}
}
