package net.sf.twip.parameterhandler;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.twip.Assume;
import net.sf.twip.internal.TwipConfigurationError;
import net.sf.twip.util.Parameter;

abstract public class AbstractNumberParameterHandler extends ParameterHandler {

	protected interface Tester {
		public boolean test(Object value);
	}

	private static final class TrueTester implements Tester {
		public boolean test(Object value) {
			return true;
		}
	}

	private static final Pattern EXPRESSION = Pattern.compile("(<|>|!)(=?)\\s*(-?\\d+)");

	private final Tester tester;

	public AbstractNumberParameterHandler(Parameter parameter) {
		super(parameter);
		this.tester = getTester();
	}

	protected abstract Comparable<? extends Number> getComparable(String numberExpression);

	private Tester getTester() {
		final Assume assumption = parameter.getAnnotation(Assume.class);
		if (assumption == null)
			return new TrueTester();

		final String expression = assumption.value();
		return getTester(expression);
	}

	protected Tester getTester(final String expression) throws TwipConfigurationError {
		final Matcher matcher = EXPRESSION.matcher(expression);
		if (!matcher.matches())
			throw new TwipConfigurationError("illegal assume expression [" + expression + "] for ["
					+ parameter.getType().getSimpleName() + "]");
		@SuppressWarnings("unchecked")
		final Comparable<Object> comparable = (Comparable) getComparable(matcher.group(3));
		if (">".equals(matcher.group(1))) {
			if ("=".equals(matcher.group(2))) {
				return new Tester() {
					public boolean test(Object value) {
						return value == null || 0 >= comparable.compareTo(value);
					}
				};
			} else {
				return new Tester() {
					public boolean test(Object value) {
						return value == null || 0 > comparable.compareTo(value);
					}
				};
			}
		} else if ("<".equals(matcher.group(1))) {
			if ("=".equals(matcher.group(2))) {
				return new Tester() {
					public boolean test(Object value) {
						return value == null || 0 <= comparable.compareTo(value);
					}
				};
			} else {
				return new Tester() {
					public boolean test(Object value) {
						return value == null || 0 < comparable.compareTo(value);
					}
				};
			}
		} else {
			if ("=".equals(matcher.group(2))) {
				return new Tester() {
					public boolean test(Object value) {
						return value == null || 0 != comparable.compareTo(value);
					}
				};
			} else {
				throw new TwipConfigurationError("illegal assume expression [" + expression + "] for ["
						+ parameter.getType().getSimpleName() + "]");
			}
		}
	}

	@Override
	public boolean test(Object value) {
		return tester.test(value);
	}
}
